"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createEmotionPlugin = void 0;
var tslib_1 = require("tslib");
var ts = tslib_1.__importStar(require("typescript"));
var source_map_1 = require("source-map");
var convert = tslib_1.__importStar(require("convert-source-map"));
var path_1 = require("path");
var lodash_1 = require("lodash");
var find_root_1 = tslib_1.__importDefault(require("find-root"));
var hashString = require('@emotion/hash').default;
var defaultEmotionModules = [
    {
        moduleName: '@emotion/css',
        exportedNames: ['css', 'keyframes', 'injectGlobal', 'cx', 'merge'],
    },
    {
        moduleName: '@emotion/styled',
        exportedNames: ['styled'],
        hasDefaultExport: true,
        styledName: 'styled',
    },
    {
        moduleName: '@emotion/react',
        exportedNames: ['css'],
    },
];
var defaultOptions = {
    sourcemap: true,
    autoLabel: true,
    labelFormat: '[local]',
    autoInject: true,
};
var getPackageRootPath = lodash_1.memoize(function (filename) { return find_root_1.default(filename); });
var hashArray = function (arr) { return hashString(arr.join('')); };
var createImportJSXAst = lodash_1.memoize(function (propertyName) {
    var importClause = ts.createImportClause(undefined, ts.createNamedImports([
        propertyName
            ? ts.createImportSpecifier(ts.createIdentifier('jsx'), ts.createIdentifier(propertyName))
            : ts.createImportSpecifier(undefined, ts.createIdentifier('jsx')),
    ]));
    var moduleSpecifier = ts.createStringLiteral('@emotion/react');
    return ts.createImportDeclaration(undefined, undefined, importClause, moduleSpecifier);
});
exports.createEmotionPlugin = function (pluginOptions) {
    var options = tslib_1.__assign(tslib_1.__assign({}, defaultOptions), pluginOptions);
    var modules = new Map(defaultEmotionModules
        .concat(options.customModules || [])
        .map(function (m) { return [m.moduleName, m]; }));
    function getImportCalls(importDeclarationNode, compilerOptions) {
        var e_1, _a;
        var importCalls = [];
        var moduleName = importDeclarationNode.moduleSpecifier
            .text;
        if (!importDeclarationNode.importClause) {
            return importCalls;
        }
        var _b = importDeclarationNode.importClause, name = _b.name, namedBindings = _b.namedBindings;
        var _loop_1 = function (moduleInfo) {
            if (moduleInfo.moduleName === moduleName ||
                (moduleInfo.includesSubPath &&
                    moduleName.includes(moduleInfo.moduleName + '/'))) {
                if (name) {
                    if (moduleInfo.hasDefaultExport) {
                        importCalls.push(tslib_1.__assign({ name: name.text, type: 'defaultImport' }, moduleInfo));
                    }
                    else if (compilerOptions.allowSyntheticDefaultImports) {
                        importCalls.push(tslib_1.__assign({ name: name.text, type: 'namespaceImport' }, moduleInfo));
                    }
                }
                if (namedBindings) {
                    if (ts.isNamedImports(namedBindings)) {
                        namedBindings.elements.forEach(function (node) {
                            if (node.propertyName
                                ? moduleInfo.exportedNames.includes(node.propertyName.text) ||
                                    (node.propertyName.text === 'default' &&
                                        moduleInfo.hasDefaultExport)
                                : moduleInfo.exportedNames.includes(node.name.text)) {
                                importCalls.push(tslib_1.__assign({ name: node.name.text, type: 'namedImport' }, moduleInfo));
                            }
                        });
                    }
                    else if (ts.isNamespaceImport(namedBindings)) {
                        importCalls.push(tslib_1.__assign({ name: namedBindings.name.text, type: 'namespaceImport' }, moduleInfo));
                    }
                }
            }
        };
        try {
            for (var _c = tslib_1.__values(modules.values()), _d = _c.next(); !_d.done; _d = _c.next()) {
                var moduleInfo = _d.value;
                _loop_1(moduleInfo);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return importCalls;
    }
    var transformer = function (context) {
        var importCalls = [];
        var compilerOptions = context.getCompilerOptions();
        var sourcemapGenerator;
        var emotionTargetClassNameCount = 0;
        var sourceFile;
        var inserted = false;
        var visitor = function (node) {
            if (ts.isSourceFile(node)) {
                inserted = false;
                return ts.visitEachChild(node, visitor, context);
            }
            if (ts.isImportDeclaration(node)) {
                importCalls = importCalls.concat(getImportCalls(node, compilerOptions));
                if (!inserted &&
                    options.autoInject &&
                    node.moduleSpecifier.text === 'react') {
                    inserted = true;
                    return [createImportJSXAst(compilerOptions.jsxFactory), node];
                }
                return node;
            }
            if (options.autoLabel || options.sourcemap) {
                if (ts.isCallExpression(node)) {
                    var expression_1 = node.expression;
                    if (ts.isCallExpression(expression_1) ||
                        ts.isPropertyAccessExpression(expression_1) ||
                        ts.isIdentifier(expression_1)) {
                        var subExpression_1 = (ts.isIdentifier(expression_1)
                            ? node
                            : expression_1).expression;
                        var transformedNode_1 = node;
                        var updateCallFunction = function () { return transformedNode_1; };
                        if (ts.isPropertyAccessExpression(expression_1)) {
                            var info = importCalls.find(function (importInfo) {
                                var expr = expression_1
                                    .expression;
                                return (importInfo.styledName ===
                                    (ts.isIdentifier(expr)
                                        ? expr.text
                                        : ts.isPropertyAccessExpression(expr)
                                            ? expr.name.text
                                            : ''));
                            });
                            if (info) {
                                expression_1 = ts.createCall(expression_1.expression, [], [ts.createStringLiteral(expression_1.name.text)]);
                            }
                        }
                        var exp_1 = ts.isCallExpression(expression_1) ? expression_1 : null;
                        if (exp_1) {
                            updateCallFunction = function () {
                                if (exp_1.arguments.length >= 1) {
                                    var filename = sourceFile.fileName;
                                    var moduleName = '';
                                    var rootPath = filename;
                                    try {
                                        rootPath = getPackageRootPath(filename);
                                        moduleName = require(rootPath + '/package.json').name;
                                    }
                                    catch (err) {
                                    }
                                    var finalPath = filename === rootPath
                                        ? 'root'
                                        : filename.slice(rootPath.length);
                                    var positionInFile = emotionTargetClassNameCount++;
                                    var stuffToHash = [moduleName];
                                    if (finalPath) {
                                        stuffToHash.push(path_1.normalize(finalPath));
                                    }
                                    else {
                                        stuffToHash.push(sourceFile.getText());
                                    }
                                    var stableClassName = "e" + hashArray(stuffToHash) + positionInFile;
                                    var _a = tslib_1.__read(exp_1.arguments, 2), el = _a[0], opts = _a[1];
                                    var targetAssignment = ts.createPropertyAssignment(ts.createIdentifier('target'), ts.createStringLiteral(stableClassName));
                                    var args = [el];
                                    args.push(ts.createObjectLiteral(opts && ts.isObjectLiteralExpression(opts)
                                        ? opts.properties.concat(targetAssignment)
                                        : [targetAssignment], true));
                                    var updatedCall = ts.updateCall(exp_1, exp_1.expression, exp_1.typeArguments, args);
                                    return ts.updateCall(transformedNode_1, updatedCall, transformedNode_1.typeArguments, transformedNode_1.arguments);
                                }
                                return transformedNode_1;
                            };
                        }
                        if (ts.isIdentifier(subExpression_1) ||
                            ts.isPropertyAccessExpression(subExpression_1)) {
                            var importedInfo = ts.isIdentifier(subExpression_1)
                                ? importCalls.find(function (imported) { return imported.name === subExpression_1.text; })
                                : importCalls.find(function (imported) {
                                    return imported.name ===
                                        subExpression_1.expression.text;
                                });
                            if (importedInfo) {
                                if (options.autoLabel) {
                                    var rawPath = sourceFile.fileName;
                                    var localNameNode = node.parent
                                        .name;
                                    if (localNameNode && ts.isIdentifier(localNameNode)) {
                                        var local = localNameNode.text;
                                        var fileName = path_1.basename(rawPath, path_1.extname(rawPath));
                                        transformedNode_1 = ts.updateCall(transformedNode_1, transformedNode_1.expression, transformedNode_1.typeArguments, transformedNode_1.arguments.concat([
                                            ts.createStringLiteral("label:" + options
                                                .labelFormat.replace('[local]', local)
                                                .replace('[filename]', fileName) + ";"),
                                        ]));
                                    }
                                }
                                if (options.sourcemap &&
                                    process.env.NODE_ENV !== 'production') {
                                    var sourceFileNode = node.getSourceFile();
                                    var lineAndCharacter = ts.getLineAndCharacterOfPosition(sourceFileNode, node.pos);
                                    var sourceFileName = path_1.relative(process.cwd(), sourceFileNode.fileName);
                                    sourcemapGenerator.addMapping({
                                        generated: {
                                            line: 1,
                                            column: 0,
                                        },
                                        source: sourceFileName,
                                        original: {
                                            line: lineAndCharacter.line + 1,
                                            column: lineAndCharacter.character + 1,
                                        },
                                    });
                                    sourcemapGenerator.setSourceContent(sourceFileName, sourceFileNode.text);
                                    var comment = convert
                                        .fromObject(sourcemapGenerator)
                                        .toComment({ multiline: true });
                                    transformedNode_1 = ts.updateCall(transformedNode_1, transformedNode_1.expression, transformedNode_1.typeArguments, transformedNode_1.arguments.concat([
                                        ts.createStringLiteral(comment),
                                    ]));
                                }
                                transformedNode_1 = ts.addSyntheticLeadingComment(transformedNode_1, ts.SyntaxKind.MultiLineCommentTrivia, '#__PURE__');
                                return updateCallFunction();
                            }
                        }
                    }
                }
            }
            return ts.visitEachChild(node, visitor, context);
        };
        return function (node) {
            sourceFile = node;
            sourcemapGenerator = new source_map_1.SourceMapGenerator({
                file: path_1.basename(node.fileName),
                sourceRoot: '',
            });
            var distNode = ts.visitNode(node, visitor);
            importCalls = [];
            return distNode;
        };
    };
    return transformer;
};
//# sourceMappingURL=index.js.map